///////////////
// IanniX
///////////////

////////////////////////////
// NxView.h
////////////////////////////

#ifndef NXVIEW_H
#define NXVIEW_H

#include <QtGui>

// IanniX Libs
#include "../base/atoms/NxTypes.h"
#include "../graphics/opengl/NxGLScore.h"

/////////////////////////////////
/////// IanniX Tab Widget ///////
class NxTabs : public QTabWidget
{
	Q_OBJECT
	
public:
  	NxTabs( QWidget* parent = 0, QTabBar* tb = 0 );
	~NxTabs();		
};
//////////////////////////////
/////// IanniX Tab Bar ///////
class NxTabBar : public QTabBar
{
  	Q_OBJECT
	
 public:
	NxTabBar( QWidget* parent = 0 );
  	void setViewer(NxView*);
	~NxTabBar();		
		
 protected slots:
	virtual void contextMenuEvent( QContextMenuEvent* );
	virtual void newScore();
  	virtual void closeTab();
  	virtual void closeAllTabs();
  	virtual void closeButActive();
  
 private:
	QWidget *mainWindow;
	NxView *viewer;
};

//////////////////////////
// Class NxView
/////////////////////////
class NxView : public QMainWindow
{
	Q_OBJECT

 public:
  	NxView( QWidget* parent = 0 );
  	~NxView() ;	
		
	QTabWidget *myTabs ;	
		
	void setCPUWarning( bool act ) ;
	void setCPUTimeStep( int val ) ;
	void setToolBarInfosTxt( QString inTxt ) ;
  	void setToolBarTime( NxGLScore* score, float inTime ) ;
	void setToolBarPlayback() ;
  	void setXMLinput( QString inFile ) ;
  	void launchGLBox( QString inFileName, bool isNew ) ;
	
	bool getAntialiasing() ;
	////////////////// UPDATE TOOGLE MENUS
	void updateCheckedMenus( bool algo, bool moveC, bool broadcM, bool visuM, bool oscifM, bool oscOutM, bool oscInM, bool snap, bool gridS, bool autostep, bool trajS, bool cursS, bool curvS, bool trigS, bool crossesS, bool circlesS, bool rectS, bool cursAS, bool infosS, bool workspaceS, bool viewtype ) ;
	////////////////// MENUBAR ACTIONS
	QAction *algoChoose, *moveCursorsChoose, *finishedCursorsChoose, *oscSendIFChoose, *oscOutModeChoose, *oscInModeChoose, *visualModeChoose, *broadcastChoose ;
	QAction *crossesShow, *curvesRectShow, *snapGrid, *autoManualStep, *circlesShow, *cursArea, *infosShow, *workspaceShow, *gridShow, *view3d, *antialias ;
	QAction *trajectsShow, *cursorsShow, *curvesShow, *triggersShow ;
	
 public slots:	
 
	virtual void changeTempo( int inTempo ) ; // must be accessed by osc /control/ message
 
	void quitDialog() ;
	void saveAndQuit() ;
  	void newScore();
  	void openSettings();
  	void closeScore();	
	void closeAll();
	void closeAllButActive();
	void closeScoreDialog() ;	// For closing Score DIALOG
	void saveAndAccept() ;		// For closing Score DIALOG
	
 protected:
 
   	//////////////////////////////////////////////// MAIN WIDGETS
  	QWidget *tmpTab;
  	QList<NxGLScore*> GLwidgets;
  	QWorkspace *myWorkspace;
  	QVBoxLayout *mainLayout;
  	QHBoxLayout *tabLayout;
  	QMenu *contextMenu;
  	QFileDialog *fileChooser;
  	QSlider *tempoSlider;
  	QDialog *opener;
	//////////////////////////////////////////////// TOOLBAR
	QString time, tempo ;
	int tbMin, tbSec, tbMs ;
	QLabel *tbTime, *tbTempo, *tbCpuValue, *tbInfoTxt ;
	QCheckBox *rTrajectsC, *rCursorsC, *rCurvesC, *rTriggersC ; // Rendering
	QLineEdit *tbTempoF ;
	QToolButton *playPause, *tbCpu ;
	QIcon cpuOnIcon, cpuOffIcon, playIcon, pauseIcon, stopIcon, resetIcon, configIcon ;
	bool cpuWarning ; // CPU warning LED
	//////////////////////////////////////////////// PREFERENCES WINDOW
	QDialog *prefMenu ;
	QLabel *nameL, *heightL, *lengthL, *IDL;
	QLabel *hostL, *outportL, *ivalueL ;
	//// FIELDS
	QLineEdit	*nameF, *heightF, *lengthF, *IDF;
	QLineEdit	*tempoF ;
	QSpinBox	*sStepF, *dStepF, *defaultSpatialPeriod ;
	QCheckBox	*manualSchStep ;
	QLineEdit	*configIPF, *hostF, *outPortF, *inPortF, *mformat1F, *mformat2F, *ivalueF;
	QLineEdit	*mapp1F, *mapp2F ;
	// Curves / Trigger OSC sending options
	QCheckBox	*oscScCID, *oscScCGID, *oscScID, *oscScGID, *oscScA, *oscScV ;	
	QCheckBox	*oscStCID, *oscStCGID, *oscStID, *oscStGID, *oscStV ;
	//// COLOR
	QPushButton *chooseScoreColor, *chooseBackColor, *chooseGridColor ;
  	QColorDialog *colorPicker ;
	Color pickedBackColor, pickedScoreColor, pickedGridColor ;
	bool backColorPicked, scoreColorPicked, gridColorPicked ;
	//////////////////////////////////////////////// COPY / PASTE
	NxObject *copied, *lastCopied ;	
	QVector<NxCursor*> cursCopied ;
	float decObjectX ;
	int incObjectID ;
	
 protected slots:
 
	virtual void giveKeyboard( QWidget* inW );
	virtual void reGiveHand() ;
	
	virtual void contextMenuEvent( QContextMenuEvent*);	
 	////////////// SCORE MENU SETTINGS
	///// TIME
	virtual void switchCirclesAlgo() ;	
	virtual void switchSendIFmessages() ;
	virtual void switchMoveCursors() ;
	virtual void switchBroadcastMode() ;
	virtual void switchVisualMode() ;
	virtual void switchOSCoutMode() ;
	virtual void switchOSCinMode() ;
	virtual void setPlaybackTime() ;
	virtual void setObjectsIPandPort() ;
	virtual void setCurvesSpatialPeriod() ;
	virtual void setCurvesBounds() ;
	virtual void setCurvesFlatten() ;
	virtual void setCursorsPlayFinished() ;
	virtual void hardOffsetsReset() ;
	///// DISPLAY - Objects Rendering
	virtual void showHideTrajects() ;
	virtual void showHideCursors() ;
	virtual void showHideCurves() ;
	virtual void showHideTriggers() ;
	///// DISPLAY
	virtual void switchAntialiasing() ;
	virtual void change2D3DView() ;
	virtual void showHideGrid() ;
	virtual void showHideScore() ;
	virtual void showHideCrosses() ;
	virtual void showHideCurvesCircles() ;
	virtual void showHideCurvesRect() ;
	virtual void showHideCursorsArea() ;
	virtual void showHideScreenInfos() ;
	virtual void switchSnapToGrid() ;
	virtual void switchAutoGridStep() ;
	virtual void resetView() ;
	virtual void setGridStep() ;

	////////////// PREFERENCES DIALOG
	virtual void showPreferences() ;
	virtual void pickBackColor() ;
	virtual void pickScoreColor() ;
	virtual void pickGridColor() ;
	virtual void setGridOpacity( int inOp ) ;
	virtual void setCursorZonesOpacity( int inOp ) ;
	virtual void setPreferences();
	////////////// PLAYBACK / TEMPO
	virtual void sendConfigOSC() ;
 	virtual void playScore() ;
	virtual void pauseScore() ;
	virtual void resetScore() ;
	virtual void resetTempo() ;
	virtual void setTbTempo() ; // Label SLOT to set Tempo 
	virtual void changeTempoTrigg( int inTempo ) ;
	////////////// CUT / COPY / PASTE / DUPLICATE //////////////
  	virtual void copy();
	virtual void cut();
	virtual void basicPaste() ;
  	virtual void paste( bool incr, bool dec ); // true to increment the ID
	virtual void duplicate();
	////////////// OPEN / SAVE
	virtual void openFromFile();
  	virtual void save();
  	virtual void saveAs();
  	virtual void saveAll();
	////////////// INFOS / HELP / MISC
	virtual void showCPUinfo() ;
	virtual void showShortcuts() ;	
  	virtual void about();
	virtual void launchWebLink( const QString & link ) ;
	virtual void launchTutorial() ;
  	virtual void font();
	virtual void languageChange();
  	virtual void showObjectEditor();
	virtual void showControl();
	virtual void showAdvanced();
	virtual void launchPd();
	virtual void launchMaxMSP();
	virtual void launchExternal();
	virtual void tmp();		// Temporary slot for undefined connections
	
  private : 
  
	int index, focusTab;
	int totalNbScore ;	// count the number of scores created since opening (to increment names)
  	QString inXMLfile;
	
	bool noScoreOpened, allClosedSaved, quitDialogMode ;
	
	bool antialiasing ;
	
	/////// SCORE VARIABLES
  	QString partName;
  	int partID, partLength, partHeight ;
	
	/////// WIDGETS INITIALISATIONS
	void initMenuBar();
  	void initToolBar();
  	void initStatusBar();
	void initCloseDialog() ;
	
	//////// CLOSE DIALOGS
	QDialog			*closeDialog ;
	QLabel			*txtL ;
	QPushButton		*quitB, *saveAndQuitB, *cancelB ;
	
	//////// DIALOG Results
  	bool settingsOk ;	// New Score
	bool closeOk ;		// Closing Score
	bool saveOk ;		// Closing Score

	//////// EXTERNALS HANDLER
	QProcess* external; // to launch PD
};

#endif

